@@ -7,13 +7,13 @@ from typing import Optional
 from pydantic import BaseModel, Field
 
 from flask_openapi3 import APIBlueprint, OpenAPI
-from flask_openapi3.models import Tag, Info
-from flask_openapi3.models.security import HTTPBearer
+from flask_openapi3 import HTTPBearer
+from flask_openapi3 import Tag, Info
 
 info = Info(title='book API', version='1.0.0')
-securitySchemes = {"jwt": HTTPBearer(bearerFormat="JWT")}
+security_schemes = {"jwt": HTTPBearer(bearerFormat="JWT")}
 
-app = OpenAPI(__name__, info=info, securitySchemes=securitySchemes)
+app = OpenAPI(__name__, info=info, security_schemes=security_schemes)
 
 tag = Tag(name='book', description="Some Book")
 security = [{"jwt": []}]
@@ -32,7 +32,7 @@ api = APIBlueprint(
     abp_security=security,
     abp_responses={"401": Unauthorized},
     # disable openapi UI
-    doc_ui=False
+    doc_ui=True
 )
 
 
@@ -50,13 +50,13 @@ def get_book():
     return {"code": 0, "message": "ok"}
 
 
-@api.post('/book')
+@api.post('/book', extra_responses={"200": {"content": {"text/csv": {"schema": {"type": "string"}}}}})
 def create_book(body: BookBody):
     assert body.age == 3
     return {"code": 0, "message": "ok"}
 
 
-@api.put('/book/<int:bid>')
+@api.put('/book/<int:bid>', operation_id='update')
 def update_book(path: Path, body: BookBody):
     assert path.bid == 1
     assert body.age == 3
