@@ -2,7 +2,7 @@
 # @Author  : llc
 # @Time    : 2021/6/21 11:23
 from flask_openapi3 import OpenAPI, OAuthConfig
-from flask_openapi3.models import Info
+from flask_openapi3 import Info
 from flask_openapi3.models.security import OAuth2, OAuthFlows, OAuthFlowImplicit
 
 info = Info(title='oauth API', version='1.0.0')
@@ -20,9 +20,9 @@ oauth2 = OAuth2(flows=OAuthFlows(
             "read:pets": "read your pets"
         }
     )))
-securitySchemes = {"oauth2": oauth2}
+security_schemes = {"oauth2": oauth2}
 
-app = OpenAPI(__name__, info=info, oauth_config=oauth_config, securitySchemes=securitySchemes)
+app = OpenAPI(__name__, info=info, oauth_config=oauth_config, security_schemes=security_schemes)
 
 security = [
     {"oauth2": ["write:pets", "read:pets"]}
