@@ -3,23 +3,25 @@
 # @Time    : 2021/5/1 21:34
 
 import inspect
+import re
 from http import HTTPStatus
-from typing import Dict, Type, Callable, List, Tuple, Any
+from typing import Dict, Type, Callable, List, Tuple, Any, ForwardRef
 
+import pydantic.typing
 from flask import Response as _Response
 from pydantic import BaseModel
 from werkzeug.routing import parse_rule
 
+from .http import HTTP_STATUS, HTTPMethod
 from .models import OPENAPI3_REF_TEMPLATE, OPENAPI3_REF_PREFIX, Tag
 from .models.common import Schema, Response, MediaType
 from .models.parameter import ParameterInType, Parameter
 from .models.path import Operation, RequestBody, PathItem
 from .models.validation_error import UnprocessableEntity
-from .status import HTTP_STATUS
 
 
-def _parse_rule(rule: str) -> str:
-    """Flask route conversion to openapi:
+def get_openapi_path(rule: str) -> str:
+    """Flask route conversion to openapi path:
     /pet/<petId> --> /pet/{petId}
     """
     uri = ''
@@ -31,32 +33,50 @@ def _parse_rule(rule: str) -> str:
     return uri
 
 
-def get_operation(func: Callable) -> Operation:
-    """Return a Operation object with summary and description."""
+def get_operation(func: Callable, *, summary: str = None, description: str = None) -> Operation:
+    """Return an Operation object with summary and description."""
     doc = inspect.getdoc(func) or ''
     doc = doc.strip()
     lines = doc.split('\n')
+    doc_summary = lines[0] or None
+    if summary is None:
+        doc_description = lines[0] if len(lines) == 0 else '</br>'.join(lines[1:]) or None
+    else:
+        doc_description = '</br>'.join(lines) or None
     operation = Operation(
-        summary=lines[0] or None,
-        description=lines[0] if len(lines) == 0 else '</br>'.join(lines[1:]) or None
+        summary=summary or doc_summary,
+        description=description or doc_description
     )
 
     return operation
 
 
-def get_func_parameter(func: Callable, arg_name='path') -> Type[BaseModel]:
+def get_operation_id_for_path(*, name: str, path: str, method: str) -> str:
+    operation_id = name + path
+    operation_id = re.sub("[^0-9a-zA-Z_]", "_", operation_id)
+    operation_id = operation_id + "_" + method.lower()
+    return operation_id
+
+
+def get_func_parameter(func: Callable, func_globals: Dict[str, Any], *, parameter_name='path') -> Type[BaseModel]:
     """Get view-func parameters.
-    arg_name has six parameters to choose from: path, query, form, body, header, cookie.
+    parameter_name has six parameters to choose from: path, query, form, body, header, cookie.
     """
     signature = inspect.signature(func)
-    p = signature.parameters.get(arg_name)
-    return p.annotation if p else None
+    param = signature.parameters.get(parameter_name)
+    annotation = param.annotation if param else None
+    if isinstance(annotation, str):
+        # PEP563
+        annotation = ForwardRef(annotation)
+        annotation = pydantic.typing.evaluate_forwardref(annotation, func_globals, func_globals)
+    return annotation
 
 
 def get_schema(obj: Type[BaseModel]) -> dict:
     """Pydantic model conversion to openapi schema"""
     assert inspect.isclass(obj) and \
-           issubclass(obj, BaseModel), f"{obj} is invalid `pydantic.BaseModel`"
+           issubclass(obj, BaseModel), f"{obj}{type(obj)} is invalid `pydantic.BaseModel`"
+
     return obj.schema(ref_template=OPENAPI3_REF_TEMPLATE)
 
 
@@ -81,6 +101,7 @@ def parse_header(header: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     if definitions:
         for name, value in definitions.items():
             components_schemas[name] = Schema(**value)
+
     return parameters, components_schemas
 
 
@@ -105,6 +126,7 @@ def parse_cookie(cookie: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     if definitions:
         for name, value in definitions.items():
             components_schemas[name] = Schema(**value)
+
     return parameters, components_schemas
 
 
@@ -129,6 +151,7 @@ def parse_path(path: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     if definitions:
         for name, value in definitions.items():
             components_schemas[name] = Schema(**value)
+
     return parameters, components_schemas
 
 
@@ -153,6 +176,7 @@ def parse_query(query: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     if definitions:
         for name, value in definitions.items():
             components_schemas[name] = Schema(**value)
+
     return parameters, components_schemas
 
 
@@ -187,6 +211,9 @@ def parse_form(form: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
     if definitions:
         for name, value in definitions.items():
             components_schemas[name] = Schema(**value)
+
+    assert content is not None, f"{form.__name__}'s properties cannot be empty."
+
     return content, components_schemas
 
 
@@ -215,13 +242,31 @@ def parse_body(body: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
     if definitions:
         for name, value in definitions.items():
             components_schemas[name] = Schema(**value)
+
+    if content is None:
+        content = {
+            "application/json": MediaType(
+                **{
+                    "schema": Schema(
+                        **schema
+                    )
+                }
+            )
+        }
+
     return content, components_schemas
 
 
-def get_responses(responses: dict, components_schemas: dict, operation: Operation) -> None:
+def get_responses(
+        responses: dict,
+        extra_responses: Dict[str, dict],
+        components_schemas: dict,
+        operation: Operation
+) -> None:
     """
     :param responses: Dict[str, BaseModel]
-    :param components_schemas: `models.component.py` Components.schemas
+    :param extra_responses: Dict[str, dict]
+    :param components_schemas: `models.component.py` `Components.schemas`
     :param operation: `models.path.py` Operation
     """
     if responses is None:
@@ -245,8 +290,9 @@ def get_responses(responses: dict, components_schemas: dict, operation: Operatio
             }
         )
         _schemas[UnprocessableEntity.__name__] = Schema(**UnprocessableEntity.schema())
-    if not responses.get("500"):
-        _responses["500"] = Response(description=HTTP_STATUS["500"])
+    # if not responses.get("500"):
+    #     _responses["500"] = Response(description=HTTP_STATUS["500"])
+    # handle extra_responses
     for key, response in responses.items():
         assert inspect.isclass(response) and \
                issubclass(response, BaseModel), f" {response} is invalid `pydantic.BaseModel`"
@@ -270,13 +316,26 @@ def get_responses(responses: dict, components_schemas: dict, operation: Operatio
         if definitions:
             for name, value in definitions.items():
                 _schemas[name] = Schema(**value)
+    # handle extra_responses
+    for key, value in extra_responses.items():
+        # key "200" value {"content":{"text/csv":{"schema":{"type": "string"}}}}
+        extra_content = value.get('content', {})
+        if extra_content:
+            # {"text/csv":{"schema":{"type": "string"}}}
+            if _responses.get(key) and isinstance(extra_content, dict):
+                _responses[key].content.update(**extra_content)  # noqa
+            else:
+                _responses[key] = Response(
+                    description=HTTP_STATUS.get(key, ""),
+                    content=extra_content
+                )
 
     components_schemas.update(**_schemas)
     operation.responses = _responses
 
 
-def validate_responses_type(responses: Dict[str, Type[BaseModel]]) -> None:
-    assert isinstance(responses, dict), "invalid `dict`"
+def validate_responses_type(responses: Dict[str, Any]) -> None:
+    assert isinstance(responses, dict), f"{responses} invalid `dict`"
 
 
 def validate_response(resp: Any, responses: Dict[str, Type[BaseModel]]) -> None:
@@ -333,6 +392,7 @@ def parse_and_store_tags(
 
 def parse_parameters(
         func: Callable,
+        *,
         components_schemas: dict = None,
         operation: Operation = None,
         doc_ui: bool = True,
@@ -343,13 +403,17 @@ def parse_parameters(
     :param operation: `models.path.py` Operation
     :param doc_ui: add openapi document UI(swagger and redoc). Defaults to True.
     """
+    func_globals = getattr(func, '__globals__', {})
+    # extra globals used in decorator(wrapper) for func, wrapper.__extra_globals__ = getattr(func, '__globals__', {})
+    func_globals.update(**getattr(func, '__extra_globals__', {}))  # noqa
+
     parameters = []
-    header = get_func_parameter(func, 'header')
-    cookie = get_func_parameter(func, 'cookie')
-    path = get_func_parameter(func, 'path')
-    query = get_func_parameter(func, 'query')
-    form = get_func_parameter(func, 'form')
-    body = get_func_parameter(func, 'body')
+    header = get_func_parameter(func, func_globals, parameter_name='header')
+    cookie = get_func_parameter(func, func_globals, parameter_name='cookie')
+    path = get_func_parameter(func, func_globals, parameter_name='path')
+    query = get_func_parameter(func, func_globals, parameter_name='query')
+    form = get_func_parameter(func, func_globals, parameter_name='form')
+    body = get_func_parameter(func, func_globals, parameter_name='body')
     if doc_ui is False:
         return header, cookie, path, query, form, body
     if header:
@@ -396,27 +460,27 @@ def parse_method(uri: str, method: str, paths: dict, operation: Operation) -> No
     :param paths: openapi doc paths
     :param operation: `models.path.py` Operation
     """
-    if method == 'get':
+    if method == HTTPMethod.GET:
         if not paths.get(uri):
             paths[uri] = PathItem(get=operation)
         else:
             paths[uri].get = operation
-    elif method == 'post':
+    elif method == HTTPMethod.POST:
         if not paths.get(uri):
             paths[uri] = PathItem(post=operation)
         else:
             paths[uri].post = operation
-    elif method == 'put':
+    elif method == HTTPMethod.PUT:
         if not paths.get(uri):
             paths[uri] = PathItem(put=operation)
         else:
             paths[uri].put = operation
-    elif method == 'patch':
+    elif method == HTTPMethod.PATCH:
         if not paths.get(uri):
             paths[uri] = PathItem(patch=operation)
         else:
             paths[uri].patch = operation
-    elif method == 'delete':
+    elif method == HTTPMethod.DELETE:
         if not paths.get(uri):
             paths[uri] = PathItem(delete=operation)
         else:
